from pathlib import Path

from celery.utils.log import get_task_logger

from portal import storage
from portal.async_celery import celery_app
from portal.async_celery.storage import submission_store_ended
from portal.service.general import find_submission

log = get_task_logger(__name__)


@celery_app.task(name='upload-submission-to-storage')
def upload_submission_to_storage(new_submission_id: str, file_params: dict):
    new_submission = find_submission(new_submission_id)
    log.info(
        f"[ASYNC] Uploading submission: {new_submission} with {file_params}")
    storage.submissions.create(entity_id=new_submission.id, **file_params)
    # TODO: Upload ended -> change state
    submission_store_ended(submission=new_submission)


@celery_app.task(name='upload-results-to-storage')
def upload_results_to_storage(new_submission_id: str, path: str):
    path = str(path)
    new_submission = find_submission(new_submission_id)
    file_params = dict(type='zip', source=dict(path=path))
    log.info(
        f"[ASYNC] Uploading result for the submission {new_submission.id} with {file_params}")
    entity = storage.results.create(entity_id=new_submission.id, **file_params)
    Path(path).unlink()
    return entity


@celery_app.task(name='clone-submission-files')
def clone_submission_files(source_id: str, target_id: str):
    source = find_submission(source_id)
    target = find_submission(target_id)
    log.info(f"[ASYNC] Cloning submission: {source} to {target}")
    storage.submissions.clone(source.id, target.id)
    submission_store_ended(target)


@celery_app.task(name='start-processing-submission')
def start_processing_submission(submission_id: str, submission_params):
    submission = find_submission(submission_id)
    log.info(f"[ASYNC] Processing submission: {submission}")
    # TODO: implement processing
    if submission_params is not None:
        print("Not implemented")
