import logging
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal.rest import rest_helpers
from portal.rest.schemas import component_schema, components_schema
from portal.service import auth, permissions
from portal.service.auth import find_client
from portal.service.components import create_component, delete_component, find_all_components, \
    update_component
from portal.service.general import find_component

components_namespace = Namespace('components')
log = logging.getLogger(__name__)


@components_namespace.route('')
class ComponentList(Resource):
    @jwt_required
    # @components_namespace.response(200, 'List of components', model=components_schema)
    @components_namespace.response(403, 'Not allowed to list components')
    def get(self):
        client = auth.find_client()
        # authorization
        permissions.require_sysadmin(client)
        components_list = find_all_components()
        return components_schema.dump(components_list)[0]

    @jwt_required
    # @components_namespace.response(201, 'Created component', model=component_schema)
    @components_namespace.response(403, 'Not allowed to add component')
    def post(self):
        client = auth.find_client()
        # authorization
        permissions.require_sysadmin(client)

        data = rest_helpers.parse_request_data(
            component_schema, action='create', resource='component'
        )
        new_component = create_component(**data)
        return component_schema.dump(new_component)[0], 201


@components_namespace.route('/<string:cid>')
@components_namespace.param('cid', 'Component id')
@components_namespace.response(404, 'Component not found')
class ComponentResource(Resource):

    @jwt_required
    # @components_namespace.response(200, 'Component', model=component_schema)
    @components_namespace.response(403, 'Not allowed to access component')
    def get(self, cid: str):
        client = find_client()
        permissions.require_sysadmin(client)

        component = find_component(cid)
        return component_schema.dump(component)[0]

    @jwt_required
    @components_namespace.response(204, 'Component deleted')
    @components_namespace.response(403, 'Not allowed to delete component')
    def delete(self, cid: str):
        client = find_client()
        # authorization
        permissions.require_sysadmin(client)

        component = find_component(cid)
        delete_component(component)
        return '', 204

    @jwt_required
    @components_namespace.response(204, 'Component updated')
    @components_namespace.response(403, 'Not allowed to update component')
    def put(self, cid: str):
        client = find_client()
        # authorization
        permissions.require_sysadmin(client)

        data = rest_helpers.parse_request_data(
            component_schema, action='update', resource='component', partial=True
        )

        component = find_component(cid)
        update_component(component, data)
        return '', 204
